package db

import (
	"applet/app/db/model"
	"applet/app/enum"
	"applet/app/utils/logx"
	"xorm.io/xorm"
)

type QrcodeWithBatchRecordsDb struct {
	Db *xorm.Engine `json:"db"`
}

func (qrcodeWithBatchRecordsDb *QrcodeWithBatchRecordsDb) Set() { // set方法
	qrcodeWithBatchRecordsDb.Db = Db
}

func (qrcodeWithBatchRecordsDb *QrcodeWithBatchRecordsDb) GetQrcodeWithBatchRecordsById(id int) (m *model.QrcodeWithBatchRecords, err error) {
	m = new(model.QrcodeWithBatchRecords)
	has, err := qrcodeWithBatchRecordsDb.Db.Where("id =?", id).Get(m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return m, nil
}

func (qrcodeWithBatchRecordsDb *QrcodeWithBatchRecordsDb) GetQrcodeWithBatchRecordsByQrcodeId(qrcodeId int) (m *model.QrcodeWithBatchRecords, err error) {
	m = new(model.QrcodeWithBatchRecords)
	has, err := qrcodeWithBatchRecordsDb.Db.Where("qrcode_id =?", qrcodeId).And("state =?", enum.QrcodeWithBatchRecordsStateForWait).Get(m)
	if err != nil {
		return nil, logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return m, nil
}

func (qrcodeWithBatchRecordsDb *QrcodeWithBatchRecordsDb) FindQrcodeWithBatchRecordsByState(state int32) (m []*model.QrcodeWithBatchRecords, err error) {
	err = qrcodeWithBatchRecordsDb.Db.Where("state =?", state).Find(&m)
	if err != nil {
		return nil, logx.Error(err)
	}
	return m, nil
}

func (qrcodeWithBatchRecordsDb *QrcodeWithBatchRecordsDb) BatchAddQrcodeWithBatchRecordsBySession(session *xorm.Session, data []*model.QrcodeWithBatchRecords) (int64, error) {
	affected, err := session.Insert(data)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func (qrcodeWithBatchRecordsDb *QrcodeWithBatchRecordsDb) FindQrcodeWithBatchRecordsById(batchId int) (m []*model.QrcodeWithBatchRecords, total int64, err error) {
	total, err = qrcodeWithBatchRecordsDb.Db.Where("batch_id =?", batchId).FindAndCount(&m)
	return
}
func (qrcodeWithBatchRecordsDb *QrcodeWithBatchRecordsDb) UpdateQrcodeWithBatchRecord(m *model.QrcodeWithBatchRecords) (int64, error) {
	affected, err := qrcodeWithBatchRecordsDb.Db.Where("id =?", m.Id).Update(m)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func (qrcodeWithBatchRecordsDb *QrcodeWithBatchRecordsDb) DeleteQrcodeWithBatchRecordsBySession(session *xorm.Session, batchId int) (delResult int64, err error) {
	m := new(model.QrcodeWithBatchRecords)
	delResult, err = session.Where("batch_id =?", batchId).Delete(m)
	return
}

type QrcodeWithBatchRecords struct {
	model.QrcodeWithBatchRecords `xorm:"extends"`
	model.Qrcode                 `xorm:"extends"`
}

func (qrcodeWithBatchRecordsDb *QrcodeWithBatchRecordsDb) FindQrcodeWithBatchRecordsLeftJoinQrcode(batchId int) (m []*QrcodeWithBatchRecords, total int64, err error) {
	total, err = qrcodeWithBatchRecordsDb.Db.Where("batch_id =?", batchId).
		Join("LEFT", "qrcode", "qrcode_with_batch_records.qrcode_id = qrcode.id").
		FindAndCount(&m)
	return
}
